﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Xml.Serialization;

namespace CG.MultiLevelXml.Serialization
{
    public enum Status
    {
        Achieved = 0,
        Terminated = 1,
        Valid = 2
    }

    [XmlType(AnonymousType = true)]
    [XmlRoot(ElementName = "data", Namespace = "", IsNullable = false)]
    public class Data
    {
        public static Data Load(string xmlFile)
        {
            var serializer = new XmlSerializer(typeof (Data));

            using( var reader = XmlReader.Create(xmlFile))
            {
                
                
                
                return (Data) serializer.Deserialize(reader);
            }
        }

        /// <summary>
        /// Drive list
        /// </summary>
        /// <remarks>
        /// No te: Use a generic collection here to 'back' the drive array.
        /// That way client code won't have to check for Drives == null
        /// </remarks>
        [XmlElement( "drive" )]
        public Drive[] Drives
        {
            get 
            {
                if( _driveList == null )
                {
                    _driveList = new List<Drive>();
                }
                return _driveList.ToArray();
            }

            set
            {
                if ( _driveList == null )
                {
                    _driveList = new List<Drive>( );
                }

                if(value != null)
                {
                    _driveList.AddRange(value);
                }
            }
        }

        private List<Drive> _driveList = new List<Drive>();
    }

    public class Drive
    {
        [XmlElement("label")]
        public string Label { get; set; }

        [XmlElement( "date" )]
        public string Date { get; set; }
        
        [XmlElement("description")]
        public string Description { get; set; }

        [XmlElement( "status" )]
        public Status Status { get; set; }

        [XmlElement( "membership" )]
        public Membership Membership { get; set; }

        /// <summary>
        /// Use this property to retrieve the MemberShip or Collection value 
        /// </summary>
        [XmlIgnore]
        public string MembershipValue
        {
            get
            {
                return IsCollection ? Membership.Collection.Value : Membership.Value;
            }
        }

        /// <summary>
        /// Use this property to determine if Membership or Collection
        /// </summary>
        [XmlIgnore]
        public bool IsCollection
        {
            get
            {
                return (Membership != null && Membership.Collection != null);
            }
        }

        /// <summary>
        /// Override to format an output string
        /// </summary>
        /// <returns></returns>
        public override string ToString( )
        {
            return String.Format("{0,-15}{1,-15}{2,-30}{3,-12}{4,-14}{5}", Label, Date, Description, Status, IsCollection, MembershipValue);
        }
    }

    public class Membership
    {
        [XmlText]
        public string Value { get; set; }

        [XmlElement( "collection", IsNullable = true)]
        public Collection Collection { get; set; }
    }

    public class Collection
    {
        [XmlText]
        public string Value { get; set; }
    }
}
